@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*                                                                        *)
 (*  Ocamlgraph: a generic graph library for OCaml                         *)
-(*  Copyright (C) 2004-2008                                               *)
+(*  Copyright (C) 2004-2010                                               *)
 (*  Sylvain Conchon, Jean-Christophe Filliatre and Julien Signoles        *)
 (*                                                                        *)
 (*  This software is free software; you can redistribute it and/or        *)
@@ -312,6 +312,31 @@ module DotAttributes : sig
 
 end
 
+(** Graph module with dot attributes *)
+module type GraphWithDotAttrs = sig
+
+  include Sig.G
+
+  val graph_attributes: t -> DotAttributes.graph list
+
+  (** Vertex attributes *)
+
+  val default_vertex_attributes: t -> DotAttributes.vertex list
+  val vertex_name : V.t -> string
+  val vertex_attributes: V.t -> DotAttributes.vertex list
+
+  (** Edge attributes *)
+
+  val default_edge_attributes: t -> DotAttributes.edge list
+  val edge_attributes: E.t -> DotAttributes.edge list
+
+  val get_subgraph : V.t -> DotAttributes.subgraph option
+    (** The box (if exists) which the vertex belongs to. Boxes with same
+	   names are not distinguished and so they should have the same
+	   attributes. *)
+
+end
+
 module Dot
   (X : sig
 
